@@ -73,6 +73,7 @@
 extern void GETPRIVMODE();
 extern void GETUSERMODE();
 extern char *inet_ntoa();
+#define NO_SLACK
 
 #elif defined(SUNOS4)
 #define HAVE_GMTOFF
@@ -88,6 +89,7 @@ char *mktemp(char *template);
 typedef int rlim_t;
 #define memmove(a,b,c) bcopy(b,a,c)
 #define NO_LINGCLOSE
+#define USE_FLOCK_SERIALIZED_ACCEPT
 
 #elif defined(SOLARIS2)
 #undef HAVE_GMTOFF
@@ -153,7 +155,9 @@ typedef int rlim_t;
 #define DEFAULT_GROUP "nobody"
 #endif
 #define DEFAULT_USER "nobody"
+#ifdef NEED_RLIM_T
 typedef int rlim_t;
+#endif
 
 #elif defined(ULTRIX)
 #define HAVE_GMTOFF
@@ -291,10 +295,6 @@ consts. Sigh. */
 extern int strcasecmp(const char *,const char *);
 extern int strncasecmp(const char *,const char *,unsigned);
 
-#elif defined(CONVEXOS)
-#define HAVE_GMTOFF
-#define NEED_STRDUP
-
 #elif defined(AUX)
 /* These are to let -Wall compile more cleanly */
 extern int strcasecmp(const char *, const char *);
@@ -316,17 +316,18 @@ extern char *crypt();
 #define NEED_STRDUP
 #define JMP_BUF sigjmp_buf
 /* fcntl() locking is expensive with NFS */
-#undef USE_FLOCK_SERIALIZED_ACCEPT
+#define USE_FLOCK_SERIALIZED_ACCEPT
 #define HAVE_SHMGET
-#define MOVEBREAK		0x4000000
 /*
  * NOTE: If when you run Apache under A/UX and you get a warning
- * that httpd couldn't move break, then the above value for
+ * that httpd couldn't move break, then the below value for
  * MOVEBREAK (64megs) is too large for your setup. Try reducing
  * to 0x2000000 which is still PLENTY of space. I doubt if
  * even on heavy systems sbrk() would be called at all...
  */
+#define MOVEBREAK		0x4000000
 #define NO_LINGCLOSE
+#define NO_SLACK
 
 #elif defined(SVR4)
 #define NO_KILLPG
@@ -409,6 +410,7 @@ extern char *crypt();
 (defined(__FreeBSD_version) && (__FreeBSD_version < 220000))
 typedef quad_t rlim_t;
 #endif
+#define USE_FLOCK_SERIALIZED_ACCEPT
 
 #elif defined(QNX)
 #ifndef crypt
@@ -485,6 +487,12 @@ typedef int rlim_t;
 
 /* Convex OS v11 */
 #elif defined(CONVEXOS11)
+#undef HAVE_GMTOFF
+#undef NO_KILLPG
+#undef NO_SETSID
+#undef NEED_STRDUP
+#define HAVE_MMAP
+
 #define NO_TIMEZONE
 #include <stdio.h>
 #include <sys/types.h>
@@ -545,7 +553,7 @@ int ap_snprintf(char *buf, size_t len, const char *format,...);
 int ap_vsnprintf(char *buf, size_t len, const char *format, va_list ap);
 #endif
 
-#if !defined(NEXT) && !defined(CONVEXOS)
+#if !defined(NEXT)
 #include <dirent.h>
 #define DIR_TYPE dirent
 #else
