@@ -200,8 +200,8 @@ void *merge_core_server_configs (pool *p, void *basev, void *virtv)
     *conf = *virt;
     if (!conf->access_name) conf->access_name = base->access_name;
     if (!conf->document_root) conf->document_root = base->document_root;
-    conf->sec = append_arrays (p, virt->sec, base->sec);
-    conf->sec_url = append_arrays (p, virt->sec_url, base->sec_url);
+    conf->sec = append_arrays (p, base->sec, virt->sec);
+    conf->sec_url = append_arrays (p, base->sec_url, virt->sec_url);
 
     return conf;
 }
@@ -886,9 +886,23 @@ const char *set_user (cmd_parms *cmd, void *dummy, char *arg)
 	else {
 	    cmd->server->server_uid = user_id;
 	    fprintf(stderr,
-		    "Warning: User directive in <VirtualHost> requires SUEXEC wrapper.\n");
+		"Warning: User directive in <VirtualHost> "
+		"requires SUEXEC wrapper.\n");
 	}
     }
+#if !defined (BIG_SECURITY_HOLE)
+    if (cmd->server->server_uid == 0) {
+	fprintf (stderr,
+"Error:\tApache has not been designed to serve pages while running\n"
+"\tas root.  There are known race conditions that will allow any\n"
+"\tlocal user to read any file on the system.  Should you still\n"
+"\tdesire to serve pages as root then add -DBIG_SECURITY_HOLE to\n"
+"\tthe EXTRA_CFLAGS line in your src/Configuration file and rebuild\n"
+"\tthe server.  It is strongly suggested that you instead modify the\n"
+"\tUser directive in your httpd.conf file to list a non-root user.\n");
+	exit (1);
+    }
+#endif
 
     return NULL;
 }
@@ -953,6 +967,11 @@ const char *set_scoreboard (cmd_parms *cmd, void *dummy, char *arg) {
     return NULL;
 }
 
+const char *set_lockfile (cmd_parms *cmd, void *dummy, char *arg) {
+    lock_fname = pstrdup (cmd->pool, arg);
+    return NULL;
+}
+
 const char *set_idcheck (cmd_parms *cmd, core_dir_config *d, int arg) {
     d->do_rfc1413 = arg;
     return NULL;
@@ -1000,10 +1019,11 @@ const char *set_max_free_servers (cmd_parms *cmd, void *dummy, char *arg) {
 const char *set_server_limit (cmd_parms *cmd, void *dummy, char *arg) {
     daemons_limit = atoi (arg);
     if (daemons_limit > HARD_SERVER_LIMIT) {
-       fprintf(stderr, "WARNING: Compile-time limit of %d servers\n",
-        HARD_SERVER_LIMIT);
-       fprintf(stderr, " Adjusting as required (to increase, please read\n");
-       fprintf(stderr, " the documentation)\n");
+       fprintf(stderr, "WARNING: MaxClients of %d exceeds compile time limit "
+           "of %d servers,\n", daemons_limit, HARD_SERVER_LIMIT);
+       fprintf(stderr, " lowering MaxClients to %d.  To increase, please "
+           "see the\n", HARD_SERVER_LIMIT);
+       fprintf(stderr, " HARD_SERVER_LIMIT define in src/httpd.h.\n");
        daemons_limit = HARD_SERVER_LIMIT;
     } else if (daemons_limit < 1) {
 	fprintf (stderr, "WARNING: Require MaxClients > 0, setting to 1\n");
@@ -1202,6 +1222,8 @@ command_rec core_cmds[] = {
     "A file for logging the server process ID"},
 { "ScoreBoardFile", set_scoreboard, NULL, RSRC_CONF, TAKE1,
     "A file for Apache to maintain runtime process management information"},
+{ "LockFile", set_lockfile, NULL, RSRC_CONF, TAKE1,
+    "The lockfile used when Apache needs to lock the accept() call"},
 { "AccessConfig", set_server_string_slot,
   (void *)XtOffsetOf (server_rec, access_confname), RSRC_CONF, TAKE1,
   "The filename of the access config file" },
@@ -1303,9 +1325,14 @@ int default_handler (request_rec *r)
       (core_dir_config *)get_module_config(r->per_dir_config, &core_module);
     int rangestatus, errstatus;
     FILE *f;
-    
+
+    /* This handler has no use for a request body (yet), but we still
+     * need to read and discard it if the client sent one.
+     */
+    if ((errstatus = discard_request_body(r)) != OK)
+        return errstatus;
+
     r->allowed |= (1 << M_GET);
-    r->allowed |= (1 << M_TRACE);
     r->allowed |= (1 << M_OPTIONS);
 
     if (r->method_number == M_INVALID) {
