@@ -138,6 +138,7 @@ gid_t group_id_list[NGROUPS_MAX];
 int max_requests_per_child;
 char *pid_fname;
 char *scoreboard_fname;
+char *lock_fname;
 char *server_argv0;
 struct in_addr bind_address;
 listen_rec *listeners;
@@ -181,6 +182,16 @@ int one_process = 0;
 #define ap_killpg(x, y)		(killpg ((x), (y)))
 #endif
 
+#if defined(USE_FCNTL_SERIALIZED_ACCEPT) || defined(USE_FLOCK_SERIALIZED_ACCEPT)
+static void expand_lock_fname(pool *p)
+{
+    char buf[20];
+
+    ap_snprintf( buf, sizeof(buf), ".%u", getpid() );
+    lock_fname = pstrcat (p, server_root_relative (p, lock_fname), buf, NULL);
+}
+#endif
+
 #if defined(USE_FCNTL_SERIALIZED_ACCEPT)
 static struct flock lock_it;
 static struct flock unlock_it;
@@ -193,8 +204,7 @@ static int lock_fd=-1;
  */
 void
 accept_mutex_init(pool *p)
-    {
-    char lock_fname[256];
+{
 
     lock_it.l_whence = SEEK_SET;   /* from current point */
     lock_it.l_start  = 0;          /* -"- */
@@ -207,19 +217,7 @@ accept_mutex_init(pool *p)
     unlock_it.l_type   = F_UNLCK;  /* set exclusive/write lock */
     unlock_it.l_pid    = 0;        /* pid not actually interesting */
 
-#ifdef __MACHTEN__
-    strncpy(lock_fname, "/var/tmp/htlock.XXXXXX", sizeof(lock_fname)-1);
-#else
-    strncpy(lock_fname, "/usr/tmp/htlock.XXXXXX", sizeof(lock_fname)-1);
-#endif
-    lock_fname[sizeof(lock_fname)-1] = '\0';
-
-    if (mktemp(lock_fname) == NULL || lock_fname[0] == '\0')
-    {
-	fprintf (stderr, "Cannot assign name to lock file!\n");
-	exit (1);
-    }
-
+    expand_lock_fname (p);
     lock_fd = popenf(p, lock_fname, O_CREAT | O_WRONLY | O_EXCL, 0644);
     if (lock_fd == -1)
     {
@@ -264,22 +262,13 @@ static int lock_fd=-1;
 void
 accept_mutex_init(pool *p)
 {
-    char lock_fname[256];
-
-    strncpy(lock_fname, "/usr/tmp/htlock.XXXXXX", sizeof(lock_fname)-1);
-    lock_fname[sizeof(lock_fname)-1] = '\0';
-    
-    if (mktemp(lock_fname) == NULL || lock_fname[0] == '\0')
-    {
-	fprintf (stderr, "Cannot assign name to lock file!\n");
-	exit (1);
-    }
 
+    expand_lock_fname (p);
     lock_fd = popenf(p, lock_fname, O_CREAT | O_WRONLY | O_EXCL, 0644);
     if (lock_fd == -1)
     {
 	perror ("open");
-	fprintf (stderr, "Cannot open lock file\n");
+	fprintf (stderr, "Cannot open lock file: %s\n", lock_fname);
 	exit (1);
     }
     unlink(lock_fname);
@@ -677,8 +666,27 @@ static void setup_shared_mem(void)
 
 #elif defined(MAP_ANON) || defined(MAP_FILE)
 /* BSD style */
+#ifdef CONVEXOS11
+    /*
+     * 9-Aug-97 - Jeff Venters (venters@convex.hp.com)
+     * ConvexOS maps address space as follows:
+     *   0x00000000 - 0x7fffffff : Kernel
+     *   0x80000000 - 0xffffffff : User
+     * Start mmapped area 1GB above start of text.
+     *
+     * Also, the length requires a pointer as the actual length is
+     * returned (rounded up to a page boundary).
+     */
+    {
+	unsigned len = SCOREBOARD_SIZE;
+	
+	m = mmap((caddr_t)0xC0000000, &len,
+		PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, NOFD, 0);
+    }
+#else
     m = mmap((caddr_t)0, SCOREBOARD_SIZE,
 	     PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
+#endif
     if (m == (caddr_t)-1)
     {
 	perror("mmap");
@@ -725,6 +733,13 @@ static void setup_shared_mem(void)
 
     if ((shmid = shmget(shmkey, SCOREBOARD_SIZE, IPC_CREAT|SHM_R|SHM_W)) == -1)
     {
+#ifdef LINUX
+	if (errno == ENOSYS) {
+	    fprintf(stderr,
+		"httpd: Your kernel was built without CONFIG_SYSVIPC\n"
+		"httpd: please consult the Apache FAQ for details\n");
+	}
+#endif
 	perror("shmget");
 	fprintf(stderr, "httpd: Could not call shmget\n");
 	exit(1);
@@ -941,6 +956,11 @@ void sync_scoreboard_image ()
 #endif
 }
 
+int exists_scoreboard_image ()
+{
+    return (scoreboard_image ? 1 : 0);
+}
+
 int update_child_status (int child_num, int status, request_rec *r)
 {
     int old_status;
@@ -1981,7 +2001,16 @@ static int make_sock(pool *pconf, const struct sockaddr_in *server)
         exit(1);
     }
 
+    /* Solaris (probably versions 2.4, 2.5, and 2.5.1 with various levels
+     * of tcp patches) has some really weird bugs where if you dup the
+     * socket now it breaks things across SIGHUP restarts.  It'll either
+     * be unable to bind, or it won't respond.
+     */
+#ifndef SOLARIS2
+    s = ap_slack(s, AP_SLACK_HIGH);
+
     note_cleanups_for_fd(pconf, s); /* arrange to close on exec or restart */
+#endif
     
 #ifndef MPE
 /* MPE does not support SO_REUSEADDR and SO_KEEPALIVE */
@@ -2050,6 +2079,12 @@ static int make_sock(pool *pconf, const struct sockaddr_in *server)
     if (ntohs(server->sin_port) < 1024) GETUSERMODE();
 #endif
     listen(s, 512);
+
+#ifdef SOLARIS2
+    s = ap_slack(s, AP_SLACK_HIGH);
+
+    note_cleanups_for_fd(pconf, s); /* arrange to close on exec or restart */
+#endif
     return s;
 }
 
@@ -2135,20 +2170,6 @@ void standalone_main(int argc, char **argv)
 	ptrans = make_sub_pool (pconf);
 
 	server_conf = read_config (pconf, ptrans, server_confname); 
-	open_logs (server_conf, pconf);
-	set_group_privs ();
-	accept_mutex_init (pconf);
-	if (!is_graceful) {
-	    reinit_scoreboard(pconf);
-	}
-#ifdef SCOREBOARD_FILE
-	else {
-	    scoreboard_fname = server_root_relative (pconf, scoreboard_fname);
-	    note_cleanups_for_fd (pconf, scoreboard_fd);
-	}
-#endif
-
-	default_server_hostnames (server_conf);
 
 	if (listeners == NULL) {
 	    if (!is_graceful) {
@@ -2183,6 +2204,22 @@ void standalone_main(int argc, char **argv)
 	    sd = -1;
 	}
 
+	init_modules (pconf, server_conf);
+	open_logs (server_conf, pconf);
+	set_group_privs ();
+	accept_mutex_init (pconf);
+	if (!is_graceful) {
+	    reinit_scoreboard(pconf);
+	}
+#ifdef SCOREBOARD_FILE
+	else {
+	    scoreboard_fname = server_root_relative (pconf, scoreboard_fname);
+	    note_cleanups_for_fd (pconf, scoreboard_fd);
+	}
+#endif
+
+	default_server_hostnames (server_conf);
+
 	set_signals ();
 	log_pid (pconf, pid_fname);
 
@@ -2273,6 +2310,8 @@ void standalone_main(int argc, char **argv)
 	}
 
 	/* we've been told to restart */
+        signal (SIGHUP, SIG_IGN);
+        signal (SIGUSR1, SIG_IGN);
 
 	if (one_process) {
 	    /* not worth thinking about */
@@ -2391,6 +2430,7 @@ main(int argc, char *argv[])
 
     suexec_enabled = init_suexec();
     server_conf = read_config (pconf, ptrans, server_confname);
+    init_modules (pconf, server_conf);
     
     if(standalone) {
         clear_pool (pconf);	/* standalone_main rereads... */
