@@ -281,8 +281,9 @@ int set_keepalive(request_rec *r)
      *   and the response status does not require a close;
      *   and the response generator has not already indicated close;
      *   and the client did not request non-persistence (Connection: close);
+     *   and    we haven't been configured to ignore the buggy twit
+     *       or they're a buggy twit coming through a HTTP/1.1 proxy
      *   and    the client is requesting an HTTP/1.0-style keep-alive
-     *          and we haven't been configured to ignore the buggy twit,
      *       or the client claims to be HTTP/1.1 compliant (perhaps a proxy);
      *   THEN we can be persistent, which requires more headers be output.
      *
@@ -304,9 +305,10 @@ int set_keepalive(request_rec *r)
         !status_drops_connection(r->status) &&
         !wimpy &&
         !find_token(r->pool, conn, "close") &&
-        (((ka_sent = find_token(r->pool, conn, "keep-alive")) &&
-          !table_get(r->subprocess_env, "nokeepalive")) ||
-         (r->proto_num >= 1001))
+	(!table_get(r->subprocess_env, "nokeepalive") ||
+	 table_get(r->headers_in, "Via")) &&
+	((ka_sent = find_token(r->pool, conn, "keep-alive")) ||
+	   (r->proto_num >= 1001))
        ) {
 	char header[256];
 	int left = r->server->keep_alive_max - r->connection->keepalives;
@@ -1041,9 +1043,17 @@ void basic_http_header (request_rec *r)
     
     if (!r->status_line)
         r->status_line = status_lines[index_of_response(r->status)];
-    
-    if (table_get(r->subprocess_env,"force-response-1.0"))
+
+    /* mod_proxy is only HTTP/1.0, so avoid sending HTTP/1.1 error response;
+     * kluge around broken browsers when indicated by force-response-1.0
+     */
+    if (r->proxyreq
+	|| (r->proto_num == 1000
+	    && table_get(r->subprocess_env,"force-response-1.0"))) {
+
 	protocol = "HTTP/1.0";
+	r->connection->keepalive = -1;
+    }
     else
 	protocol = SERVER_PROTOCOL;
 
@@ -1087,34 +1097,31 @@ static void terminate_header (BUFF *client)
     bputs("\015\012", client);    /* Send the terminating empty line */
 }
 
+/* Build the Allow field-value from the request handler method mask.
+ * Note that we always allow TRACE, since it is handled below.
+ */
 static char *make_allow(request_rec *r)
 {
-    int allowed = r->allowed;
-
-    if( allowed == 0 ) {
-	/* RFC2068 #14.7, Allow must contain at least one method.  So rather
-	 * than deal with the possibility of trying not to emit an Allow:
-	 * header, i.e. #10.4.6 says 405 Method Not Allowed MUST include
-	 * an Allow header, we'll just say TRACE is valid.
-	 */
-	return( "TRACE" );
-    }
-
-    return 2 + pstrcat(r->pool, (allowed & (1 << M_GET)) ? ", GET, HEAD" : "",
-		       (allowed & (1 << M_POST)) ? ", POST" : "",
-		       (allowed & (1 << M_PUT)) ? ", PUT" : "",
-		       (allowed & (1 << M_DELETE)) ? ", DELETE" : "",
-		       (allowed & (1 << M_OPTIONS)) ? ", OPTIONS" : "",
-		       (allowed & (1 << M_TRACE)) ? ", TRACE" : "",
-		       NULL);
-    
+    return 2 + pstrcat(r->pool,
+                       (r->allowed & (1 << M_GET)) ? ", GET, HEAD" : "",
+                       (r->allowed & (1 << M_POST)) ? ", POST" : "",
+                       (r->allowed & (1 << M_PUT)) ? ", PUT" : "",
+                       (r->allowed & (1 << M_DELETE)) ? ", DELETE" : "",
+                       (r->allowed & (1 << M_OPTIONS)) ? ", OPTIONS" : "",
+                       ", TRACE",
+                       NULL);
 }
 
 int send_http_trace (request_rec *r)
 {
+    int rv;
+
     /* Get the original request */
     while (r->prev) r = r->prev;
 
+    if ((rv = setup_client_block(r, REQUEST_NO_BODY)))
+        return rv;
+
     hard_timeout("send TRACE", r);
 
     r->content_type = "message/http";
@@ -1345,7 +1352,7 @@ int setup_client_block (request_rec *r, int read_policy)
 
 int should_client_block (request_rec *r)
 {
-    if (is_HTTP_ERROR(r->status))
+    if (r->read_length || is_HTTP_ERROR(r->status))
         return 0;
 
     if (!r->read_chunked && (r->remaining <= 0))
@@ -1447,7 +1454,11 @@ long get_client_block (request_rec *r, char *buffer, int bufsiz)
             buffer[chunk_start++] = LF;
             buffer += chunk_start;       /* and pass line on to caller   */
             bufsiz -= chunk_start;
-        }
+        } else {
+	    /* REQUEST_CHUNKED_DECHUNK -- do not include the length of
+	     * the header in the return value */
+	    chunk_start = 0;
+	}
     }
                                      /* When REQUEST_CHUNKED_PASS, we are */
     if (r->remaining == -1) {        /* reading footers until empty line  */
@@ -1514,6 +1525,43 @@ long get_client_block (request_rec *r, char *buffer, int bufsiz)
     return (chunk_start + len_read);
 }
 
+/* In HTTP/1.1, any method can have a body.  However, most GET handlers
+ * wouldn't know what to do with a request body if they received one.
+ * This helper routine tests for and reads any message body in the request,
+ * simply discarding whatever it receives.  We need to do this because
+ * failing to read the request body would cause it to be interpreted
+ * as the next request on a persistent connection.  
+ *
+ * Since we return an error status if the request is malformed, this
+ * routine should be called at the beginning of a no-body handler, e.g.,
+ *
+ *    if ((retval = discard_request_body(r)) != OK)
+ *        return retval;
+ */
+int discard_request_body(request_rec *r)
+{
+    int rv;
+
+    if ((rv = setup_client_block(r, REQUEST_CHUNKED_PASS)))
+        return rv;
+
+    if (should_client_block(r)) {
+        char dumpbuf[HUGE_STRING_LEN];
+
+        hard_timeout("reading request body", r);
+        while ((rv = get_client_block(r, dumpbuf, HUGE_STRING_LEN)) > 0)
+            continue;
+        kill_timeout(r);
+
+        if (rv < 0)
+            return HTTP_BAD_REQUEST;
+    }
+    return OK;
+}
+
+/*
+ * Send the body of a response to the client.
+ */
 long send_fd(FILE *f, request_rec *r) { return send_fd_length(f, r, -1); }
 
 long send_fd_length(FILE *f, request_rec *r, long length)
