@@ -85,6 +85,24 @@
  * they change, all the way down.
  */
 
+
+/*
+ * We don't want people able to serve up pipes, or unix sockets, or other
+ * scary things.  Note that symlink tests are performed later.
+ */
+static int check_safe_file(request_rec *r)
+{
+    if (r->finfo.st_mode == 0		/* doesn't exist */
+	|| S_ISDIR (r->finfo.st_mode)
+	|| S_ISREG (r->finfo.st_mode)
+	|| S_ISLNK (r->finfo.st_mode)) {
+	return OK;
+    }
+    log_reason("object is not a file, directory or symlink", r->filename, r);
+    return HTTP_FORBIDDEN;
+}
+
+
 int check_symlinks (char *d, int opts)
 {
     struct stat lfi, fi;
@@ -310,17 +328,23 @@ int directory_walk (request_rec *r)
     if (res != OK) {
 	return res;
     }
-    
+
+    if ((res = check_safe_file(r))) {
+	return res;
+    }
+
     if (test_filename[strlen(test_filename)-1] == '/')
 	--num_dirs;
 
-    if (S_ISDIR (r->finfo.st_mode)) ++num_dirs;
+    if (S_ISDIR (r->finfo.st_mode)) {
+	++num_dirs;
+    }
 
     for (i = 1; i <= num_dirs; ++i) {
         core_dir_config *core_dir =
 	  (core_dir_config *)get_module_config(per_dir_defaults, &core_module);
 	int overrides_here;
-        void *this_conf = NULL, *htaccess_conf = NULL;
+        void *this_conf, *htaccess_conf = NULL;
 	char *this_dir = make_dirstr (r->pool, test_filename, i);
 	int j;
       
@@ -349,6 +373,7 @@ int directory_walk (request_rec *r)
 	      (core_dir_config *)get_module_config(entry_config, &core_module);
 	    entry_dir = entry_core->d;
 	
+	    this_conf = NULL;
 	    if (entry_core->r) {
 		if (!regexec(entry_core->r, this_dir, 0, NULL,
 			     (j == num_sec) ? 0 : REG_NOTEOL)) {
@@ -399,8 +424,16 @@ int directory_walk (request_rec *r)
 
     r->per_dir_config = per_dir_defaults;
 
-    if ((res = check_symlinks (r->filename, allow_options(r))))
-    {
+    /* Symlink permissions are determined by the parent.  If the request is for
+     * a directory then applying the symlink test here would use the
+     * permissions of the directory as opposed to its parent.  Consider a
+     * symlink pointing to a dir with a .htaccess disallowing symlinks.  If you
+     * access /symlink (or /symlink/) you would get a 403 without this S_ISDIR
+     * test.  But if you accessed /symlink/index.html, for example, you would
+     * *not* get the 403.
+     */
+    if (!S_ISDIR (r->finfo.st_mode)
+	&& (res = check_symlinks (r->filename, allow_options(r)))) {
 	log_reason("Symbolic link not allowed", r->filename, r);
 	return res;
     }
@@ -669,24 +702,38 @@ request_rec *sub_req_lookup_file (const char *new_file, const request_rec *r)
 	    rnew->finfo.st_mode = 0;
 	}
 
+	if ((res = check_safe_file(rnew))) {
+	    rnew->status = res;
+	    return rnew;
+	}
+
 	rnew->per_dir_config = r->per_dir_config;
 
-	if ((res = check_symlinks (rnew->filename, allow_options (rnew)))) {
-	    log_reason ("Symbolic link not allowed", rnew->filename, rnew);
-	    rnew->status = res;
-	    return rnew;
-	}
-	/* do a file_walk, if it doesn't change the per_dir_config then
-	 * we know that we don't have to redo all the access checks */
-	if ((res = file_walk (rnew))) {
-	    rnew->status = res;
-	    return rnew;
-	}
-	if (rnew->per_dir_config == r->per_dir_config) {
-	    if ((res = find_types (rnew)) || (res = run_fixups (rnew))) {
+	/* no matter what, if it's a subdirectory, we need to re-run
+	 * directory_walk */
+	if (S_ISDIR (rnew->finfo.st_mode)) {
+	    res = directory_walk (rnew);
+	    if (!res) {
+		res = file_walk (rnew);
+	    }
+	} else {
+	    if ((res = check_symlinks (rnew->filename, allow_options (rnew)))) {
+		log_reason ("Symbolic link not allowed", rnew->filename, rnew);
 		rnew->status = res;
+		return rnew;
+	    }
+	    /* do a file_walk, if it doesn't change the per_dir_config then
+	     * we know that we don't have to redo all the access checks */
+	    if ((res = file_walk (rnew))) {
+		rnew->status = res;
+		return rnew;
+	    }
+	    if (rnew->per_dir_config == r->per_dir_config) {
+		if ((res = find_types (rnew)) || (res = run_fixups (rnew))) {
+		    rnew->status = res;
+		}
+		return rnew;
 	    }
-	    return rnew;
 	}
     } else {
 	/* XXX: this should be set properly like it is in the same-dir case
@@ -873,8 +920,10 @@ void process_request_internal (request_rec *r)
          * we handle it specially.
          */
         if (r->method_number == M_TRACE) {
-            send_http_trace(r);
-            finalize_request_protocol(r);
+            if ((access_status = send_http_trace(r)))
+	        die(access_status, r);
+            else
+                finalize_request_protocol(r);
             return;
         }
 
@@ -898,6 +947,10 @@ void process_request_internal (request_rec *r)
 	return;
     }
 
+    if (r->proto_num > 1000 && table_get (r->subprocess_env, "downgrade-1.0")) {
+	r->proto_num = 1000;
+    }
+
     /* NB: directory_walk() clears the per_dir_config, so we don't inherit from
        location_walk() above */
 
@@ -1058,6 +1111,8 @@ request_rec *internal_internal_redirect (const char *new_uri, request_rec *r)
 				  */
     new->no_local_copy = r->no_local_copy;
 
+    new->read_length = r->read_length;  /* We can only read it once */
+
     ap_snprintf (t, sizeof(t), "%d", r->status);
     table_set (new->subprocess_env, "REDIRECT_STATUS", pstrdup (r->pool, t));
 
