@@ -126,6 +126,7 @@
 #endif
 #define DEFAULT_PIDLOG "logs/httpd.pid"
 #define DEFAULT_SCOREBOARD "logs/apache_runtime_status"
+#define DEFAULT_LOCKFILE "logs/accept.lock"
 
 /* Define this to be what your HTML directory content files are called */
 #define DEFAULT_INDEX "index.html"
@@ -255,19 +256,25 @@
  * Example: "Apache/1.1.0 MrWidget/0.1-alpha" 
  */
 
-#define SERVER_BASEVERSION "Apache/1.2.0" /* SEE COMMENTS ABOVE */
+#define SERVER_BASEVERSION "Apache/1.2.4" /* SEE COMMENTS ABOVE */
 #ifdef SERVER_SUBVERSION
 #define SERVER_VERSION	SERVER_BASEVERSION " " SERVER_SUBVERSION
 #else
 #define SERVER_VERSION	SERVER_BASEVERSION
 #endif
 
+/* Numeric release version identifier: major minor bugfix betaseq
+ * Always increases along the same track as the source branch.
+ */
+#define APACHE_RELEASE 1020499
+
 #define SERVER_PROTOCOL "HTTP/1.1"
 #define SERVER_SUPPORT "http://www.apache.org/"
 
 #define DECLINED -1		/* Module declines to handle */
 #define OK 0			/* Module has handled this stage. */
 
+
 /* ----------------------- HTTP Status Codes  ------------------------- */
 
 #define RESPONSE_CODES 38
@@ -711,3 +718,31 @@ char *get_local_host(pool *);
 unsigned long get_virthost_addr (const char *hostname, unsigned short *port);
 
 extern time_t restart_time;
+
+/*
+ * Apache tries to keep all of its long term filehandles (such as log files,
+ * and sockets) above this number.  This is to workaround problems in many
+ * third party libraries that are compiled with a small FD_SETSIZE.  There
+ * should be no reason to lower this, because it's only advisory.  If a file
+ * can't be allocated above this number then it will remain in the "slack"
+ * area.
+ *
+ * Only the low slack line is used by default.  If HIGH_SLACK_LINE is defined
+ * then an attempt is also made to keep all non-FILE * files above the high
+ * slack line.  This is to work around a Solaris C library limitation, where it
+ * uses an unsigned char to store the file descriptor.
+ */
+#ifndef LOW_SLACK_LINE
+#define LOW_SLACK_LINE	15
+#endif
+/* #define HIGH_SLACK_LINE	255 */
+
+/*
+ * The ap_slack() function takes a fd, and tries to move it above the indicated
+ * line.  It returns an fd which may or may not have moved above the line, and
+ * never fails.  If the high line was requested and it fails it will also try
+ * the low line.
+ */
+int ap_slack (int fd, int line);
+#define AP_SLACK_LOW	1
+#define AP_SLACK_HIGH	2
