@@ -1,6 +1,5 @@
-
 /* ====================================================================
- * Copyright (c) 1995 The Apache Group.  All rights reserved.
+ * Copyright (c) 1995-1997 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -52,30 +51,17 @@
  */
 
 /*
- * The @ and # characters in the following comment are used by the Debian
- * package creation procedure to automatically extract a short description
- * and a full doc. for the module (the lines between # (with the @ lines
- * removed) go to the short description, the lines between @ (without any
- * of the # lines) go to the doc).
- *
- */
-
-/*
- * @
- * #
- * mod_auth_dbm: authentication using DBM files.
- *
- * If auth_db_module (mod_auth_db) is usable and you have DB files,
- * prefer it (cf. Apache README).
- * #
- *
- * Provides the AuthDBMUserFile and AuthDBMGroupFile commands to
- * control authentication.
- *
+ * http_auth: authentication
+ * 
  * Rob McCool & Brian Behlendorf.
  * 
- * Adapted to Shambhala by rst.
- * @
+ * Adapted to Apache by rst.
+ *
+ * dirkx - Added Authoritative control to allow passing on to lower  
+ *         modules if and only if the user-id is not known to this
+ *         module. A known user with a faulty or absent password still
+ *         causes an AuthRequired. The default is 'Authoritative', i.e.
+ *         no control is passed along.
  */
 
 #include "httpd.h"
@@ -89,12 +75,28 @@ typedef struct  {
 
     char *auth_dbmpwfile;
     char *auth_dbmgrpfile;
+    int   auth_dbmauthoritative;
 
 } dbm_auth_config_rec;
 
 void *create_dbm_auth_dir_config (pool *p, char *d)
 {
-    return pcalloc (p, sizeof(dbm_auth_config_rec));
+    dbm_auth_config_rec *sec
+       = (dbm_auth_config_rec *)pcalloc (p, sizeof(dbm_auth_config_rec));
+
+    sec->auth_dbmpwfile = NULL;
+    sec->auth_dbmgrpfile = NULL;
+    sec->auth_dbmauthoritative = 1; /* fortress is secure by default */
+
+    return sec;
+}
+
+const char *set_dbm_slot (cmd_parms *cmd, void *offset, char *f, char *t)
+{
+    if (!t || strcmp(t, "dbm"))
+	return DECLINE_CMD;
+
+    return set_string_slot(cmd, offset, f);
 }
 
 command_rec dbm_auth_cmds[] = {
@@ -104,6 +106,15 @@ command_rec dbm_auth_cmds[] = {
 { "AuthDBMGroupFile", set_string_slot,
     (void*)XtOffsetOf(dbm_auth_config_rec, auth_dbmgrpfile),
     OR_AUTHCFG, TAKE1, NULL },
+{ "AuthUserFile", set_dbm_slot,
+    (void*)XtOffsetOf(dbm_auth_config_rec, auth_dbmpwfile),
+    OR_AUTHCFG, TAKE12, NULL },
+{ "AuthGroupFile", set_dbm_slot,
+    (void*)XtOffsetOf(dbm_auth_config_rec, auth_dbmgrpfile),
+    OR_AUTHCFG, TAKE12, NULL },
+{ "AuthDBMAuthoritative", set_flag_slot,
+    (void*)XtOffsetOf(dbm_auth_config_rec, auth_dbmauthoritative),
+    OR_AUTHCFG, FLAG, "Set to 'no' to allow access control to be passed along to lower modules, if the UserID is not known in this module" },
 { NULL }
 };
 
@@ -176,7 +187,9 @@ int dbm_authenticate_basic_user (request_rec *r)
         return DECLINED;
 	
     if(!(real_pw = get_dbm_pw(r, c->user, sec->auth_dbmpwfile))) {
-        sprintf(errstr,"DBM user %s not found", c->user);
+	if (!(sec->auth_dbmauthoritative))
+	    return DECLINED;
+        ap_snprintf(errstr, sizeof(errstr), "DBM user %s not found", c->user);
 	log_reason (errstr, r->filename, r);
 	note_basic_auth_failure (r);
 	return AUTH_REQUIRED;
@@ -186,7 +199,8 @@ int dbm_authenticate_basic_user (request_rec *r)
     if (colon_pw) *colon_pw='\0';   
     /* anyone know where the prototype for crypt is? */
     if(strcmp(real_pw,(char *)crypt(sent_pw,real_pw))) {
-        sprintf(errstr,"user %s: password mismatch",c->user);
+        ap_snprintf(errstr, sizeof(errstr), 
+		"user %s: password mismatch",c->user);
 	log_reason (errstr, r->uri, r);
 	note_basic_auth_failure (r);
 	return AUTH_REQUIRED;
@@ -208,7 +222,8 @@ int dbm_check_auth(request_rec *r) {
     require_line *reqs = reqs_arr ? (require_line *)reqs_arr->elts : NULL;
 
     register int x;
-    char *t, *w;
+    const char *t;
+    char *w;
 
     if (!sec->auth_dbmgrpfile) return DECLINED;
     if (!reqs_arr) return DECLINED;
@@ -221,11 +236,15 @@ int dbm_check_auth(request_rec *r) {
         w = getword(r->pool, &t, ' ');
 	
         if(!strcmp(w,"group") && sec->auth_dbmgrpfile) {
-           char *orig_groups,*groups,*v;
+           const char *orig_groups,*groups;
+	   char *v;
 
            if (!(groups = get_dbm_grp(r, user, sec->auth_dbmgrpfile))) {
-               sprintf(errstr,"user %s not in DBM group file %s",
-		       user, sec->auth_dbmgrpfile);
+	       if (!(sec->auth_dbmauthoritative))
+	           return DECLINED;
+               ap_snprintf(errstr, sizeof(errstr), 
+			"user %s not in DBM group file %s",
+			user, sec->auth_dbmgrpfile);
 	       log_reason (errstr, r->filename, r);
 	       note_basic_auth_failure (r);
 	       return AUTH_REQUIRED;
@@ -240,7 +259,8 @@ int dbm_check_auth(request_rec *r) {
                        return OK;
                }
            }
-           sprintf(errstr,"user %s not in right group",user);
+           ap_snprintf(errstr, sizeof(errstr), 
+		"user %s not in right group",user);
 	   log_reason (errstr, r->filename, r);
            note_basic_auth_failure(r);
 	   return AUTH_REQUIRED;
@@ -266,5 +286,6 @@ module dbm_auth_module = {
    NULL,			/* check access */
    NULL,			/* type_checker */
    NULL,			/* fixups */
-   NULL				/* logger */
+   NULL,			/* logger */
+   NULL				/* header parser */
 };
